
/**
 * Main WindowController class for Recursion lab
 * reference solution/starter code.  Specify
 * and create Sierpinski Gasket and variegated
 * stairs objects in response to mouse press
 * and move events.
 * 
 * Note: you do not need to modify this class to satisfy
 * the base requirements.  If you do not modify it, you
 * do not need to submit it.  However, if you do modify
 * it you should document it properly and include it
 * in your submission.
 * 
 * @author Jim Teresco
 * The College of Saint Rose, Fall 2013
 * Updated Spring 2014 based on submission by Scott Whitney and Mike Gallo
 * 
 * $Id: Recursion.java 2235 2013-10-30 18:55:53Z terescoj $
 */

import objectdraw.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

public class Recursion extends WindowController implements ActionListener {

    // instance variables dealing with drawing of gaskets
    private Location gasketCorner1;
    private Location gasketCorner2;
    private Line gasketLine1;
    private Line gasketLine2;
    private Line gasketLine3;
    private int gasketCornerCount;

    // instance variables dealing with variegated stairs
    private Location stairsCorner;
    private FramedRect stairsOutline;
    private boolean drawingStairs;

    // our JRadioButtons so we know what to start drawing on a mouse press
    private JRadioButton gasketButton;
    private JRadioButton stairsButton;

    public void begin() 
    {
        // set up the simple GUI
        Container contentPane = getContentPane();
        JPanel panel = new JPanel();
  
        // placing our JRadioButtons in this ButtonGroup
        // will allow them to behave properly as a
        // radio group (only one can be selected).
        ButtonGroup group = new ButtonGroup();
        gasketButton = new JRadioButton("Gasket");
        gasketButton.setSelected(true);
        stairsButton = new JRadioButton("Stairs");
        group.add(gasketButton);
        group.add(stairsButton);

        // put them in the panel and the panel in the south
        panel.add(gasketButton);
        panel.add(stairsButton);
        contentPane.add(panel, BorderLayout.SOUTH);
        contentPane.validate();

        // Add action listeners so we can abort operations
	// part way through if we switch
        gasketButton.addActionListener(this);
        stairsButton.addActionListener(this);

        // we're not drawing anything yet when we start
        gasketCornerCount = 0;
        drawingStairs = false;
    }

    // start, continue, or complete the specification of the 
    // points/size of one of our recursive objects
    public void onMousePress(Location point) {

        if (gasketButton.isSelected()) {
            
            if (gasketCornerCount == 0) {
                
                // the first phase is to remember the press point and 
                // to create a "rubber banding" line as the mouse moves
                // to select the second point
                gasketCorner1 = point;
                gasketCornerCount = 1;
                gasketLine1 = new Line(point, point, canvas);
            }
            else if (gasketCornerCount == 1) {
                
                // the second phase leaves our rubber banding line
                // in place, remembers the second point, and sets
                // up two more rubber banding lines that will form
                // a rubber banding triangle
                gasketLine1.setEnd(point);
                gasketCorner2 = point;
                gasketLine2 = new Line(gasketCorner1, point, canvas);
                gasketLine3 = new Line(point, point, canvas);
                gasketCornerCount = 2;
            }
            else if (gasketCornerCount == 2) {
                
                // we've selected the third point, so remove our lines
                // and create a gasket!
                gasketLine1.removeFromCanvas();
                gasketLine2.removeFromCanvas();
                gasketLine3.removeFromCanvas();
                gasketLine1 = null;
                gasketLine2 = null;
                gasketLine3 = null;
                new Sierpinski(gasketCorner1, gasketCorner2, point, canvas);
                gasketCornerCount = 0;
            }
        }
        else {
            if (drawingStairs) {
                // create an appropriate stairs if valid
                double size = stairsOutline.getWidth();
                stairsOutline.removeFromCanvas();
                stairsOutline = null;
                if (size > 0) {
                    new Stairs(stairsCorner, size, canvas);
                }
                drawingStairs = false;
            }
            else {
                // start drawing the outline for some new stairs
                stairsCorner = point;
                stairsOutline = new FramedRect(point, 0, 0, canvas);
                drawingStairs = true;
            }
        }
    }

    // show intermediate lines/triangles/squares as we specify
    // corners/endpoints
    public void onMouseMove(Location point) {

        if (gasketButton.isSelected()) {
            if (gasketCornerCount == 1) {
                // during this phase, we "rubber band" one line
                gasketLine1.setEnd(point);
            }
            else if (gasketCornerCount == 2) {
                // during this phase we rubber band two lines
                // for the rubber banding triangle effect
                gasketLine2.setEnd(point);
                gasketLine3.setEnd(point);
            }
        }
        else {
            if (drawingStairs) {
                // compute a width, height between start point and this point
                // update rectangle as appropriate
                double dx = point.getX() - stairsCorner.getX();
                double dy = point.getY() - stairsCorner.getY();
                double size = Math.max(dx, dy);
                if (size < 0) size = 0;
                stairsOutline.setWidth(size);
                stairsOutline.setHeight(size);
            }
        }
    }

    /**
     * Clean up the partial figures if we switch midway.
     */
    public void actionPerformed(ActionEvent e) {
        if(e.getSource() == gasketButton) {
            stairsCorner = null;
            if(stairsOutline != null) {
                stairsOutline.removeFromCanvas();
                stairsOutline = null;
            }
            drawingStairs = false;
        }
        if(e.getSource() == stairsButton) {
            if(gasketLine1 != null) {
                gasketLine1.removeFromCanvas();
                gasketLine1 = null;
            }
            if(gasketLine2 != null) {
                gasketLine2.removeFromCanvas();
                gasketLine2 = null;
            }
            if(gasketLine3 != null) {
                gasketLine3.removeFromCanvas();
                gasketLine3 = null;
            }
            gasketCornerCount = 0;
	    drawingStairs = false;
        }
    }
}
